module Codebreaker
  class Game
    attr_reader(
      :secret,
      :attempts_total,
      :attempts,
      :hints_available,
      :hints_used,
      :hints_given,
      :result,
      :level_name
    )

    DIFFICULTIES = {
      low: { attempts: 15, hints: 2 },
      medium: { attempts: 10, hints: 2 },
      hell: { attempts: 5, hints: 1 }
    }.freeze

    def initialize(level)
      @secret = Array.new(4) { rand(1..6).to_s }
      @level_name = level.to_s
      @attempts_total = DIFFICULTIES[level][:attempts]
      @attempts = DIFFICULTIES[level][:attempts]
      @hints_available = @secret.sample(DIFFICULTIES[level][:hints])
      @hints_given = []
    end

    private

    def check_full_matches
      @secret.map.with_index do |number, index|
        next number unless number == @guess[index]

        @result += '+'
        @guess[index] = nil
      end
    end

    def check_partial_matches(rest)
      @guess.compact.each do |number|
        index = rest.index(number)
        next unless index

        @result += '-'
        rest[index] = nil
      end
    end

    public

    def compare_with(guess)
      @guess = guess.chars
      @result = ''

      rest = check_full_matches
      check_partial_matches(rest)
      @attempts -= 1
      @result
    end

    def receive_hint
      return unless @hints_available.length.positive?

      @hints_given << @hints_available.pop

      @hints_given.last
    end

    def win?(guess)
      @secret == guess.chars
    end

    def lose?
      !@attempts.positive?
    end
  end
end
